/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.jsf;

import java.util.HashMap;
import java.util.Map;

import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;

import javax.portlet.PortletRequest;

import javax.servlet.http.HttpServletRequest;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.PolicyEnforcementPoint;
import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.jee.provisioning.HttpServletProvisioningServicePoint;
import net.sf.jguard.jsf.authentication.JSFAuthenticationBindings;
import net.sf.jguard.jsf.authorization.JSFAuthorizationBindings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * JSF PhaseListener implementation to control in one unique point all access.
 * this class do a bridge between JSF and jGuard and its PolicyEnforcementPoint.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 *
 */
public class AccessListener implements PhaseListener {

	private static final long serialVersionUID = 2813722561916091752L;
	private static PolicyEnforcementPoint pep = null; 
	private static boolean initialized;
	private static final String LISTENER_CONFIGURATION_LOCATION = "listenerConfigurationLocation";
	public final static String FACES_CONTEXT="facesContext";
	
        private static final Logger logger = LoggerFactory.getLogger(AccessListener.class.getName());
        
	public void afterPhase(PhaseEvent event) {
            logger.debug(" after phase "+event.getPhaseId());
            if(PhaseId.RESTORE_VIEW==event.getPhaseId()||
               PhaseId.INVOKE_APPLICATION==event.getPhaseId()){
                AccessContext context = new AccessContext();
		context.setAttribute(FACES_CONTEXT, event.getFacesContext());
		pep.process(context);
            }
	}

	public void beforePhase(PhaseEvent event) {
            logger.debug(" before phase "+event.getPhaseId());
            if(!initialized){
                    initialize(event);
                    initialized = true;
            }
	}
	
	private static void initialize(PhaseEvent event) {
		FacesContext fc = event.getFacesContext();
		
		//init parameter is now in the context
		String authenticationBindingsImpl = fc.getExternalContext().getInitParameter(CoreConstants.AUTHENTICATION_BINDINGS);
		if(authenticationBindingsImpl == null ||authenticationBindingsImpl.equals("")){
			authenticationBindingsImpl = JSFAuthenticationBindings.class.getName();
		}
		
		Map options = new HashMap();
		
		options.put(CoreConstants.AUTHENTICATION_BINDINGS,authenticationBindingsImpl);
		
		//init parameter is now in the context
		String filterConfigurationLocation = ExternalContextUtil.getContextPath(fc.getExternalContext(), fc.getExternalContext().getInitParameter(LISTENER_CONFIGURATION_LOCATION));
		options.put(CoreConstants.CONFIGURATION_LOCATION,filterConfigurationLocation);
		
		String authenticationScope = fc.getExternalContext().getInitParameter(SecurityConstants.AUTHENTICATION_SCOPE);
		if(authenticationScope==null || "".equals(authenticationScope)){
			authenticationScope = CoreConstants.LOCAL_SCOPE;
		}
		options.put(CoreConstants.AUTHENTICATION_SCOPE,authenticationScope);
		
		//PolicyDecisionPoint init parameters
		//init parameter is now in the external context
		String authorizationBindingsImpl = fc.getExternalContext().getInitParameter(CoreConstants.AUTHORIZATION_BINDINGS);
		if(authorizationBindingsImpl == null ||authorizationBindingsImpl.equals("")){
			authorizationBindingsImpl = JSFAuthorizationBindings.class.getName();
		}
		options.put(CoreConstants.AUTHORIZATION_BINDINGS,authorizationBindingsImpl);
		
		//init parameter is now in the external context
		String provisioningServicePointImpl = fc.getExternalContext().getInitParameter(CoreConstants.PROVISIONING_SERVICE_POINT);
		if(provisioningServicePointImpl == null ||provisioningServicePointImpl.equals("")){
			provisioningServicePointImpl = HttpServletProvisioningServicePoint.class.getName();
		}
		options.put(CoreConstants.PROVISIONING_SERVICE_POINT,provisioningServicePointImpl);
		
                String applicationName = fc.getExternalContext().getInitParameter(CoreConstants.APPLICATION_NAME);
                if(applicationName==null || "".equals(applicationName)){
                    Object request = fc.getExternalContext().getRequest();
                   if(HttpServletRequest.class.isAssignableFrom(request.getClass())){
                        applicationName = ((HttpServletRequest)request).getSession(true).getServletContext().getServletContextName();
                    }else if(PortletRequest.class.isAssignableFrom(request.getClass())){
                        applicationName = ((PortletRequest)request).getPortletSession(true).getPortletContext().getPortletContextName();
                    }else{
                        throw new RuntimeException(" applicationName is null or empty ");
                    }
                }
                options.put(CoreConstants.APPLICATION_NAME,applicationName);

		pep = new PolicyEnforcementPoint(options);
	}
        
        /**
         * we can select only one phase or all phases 
         * and we want to be called after  the RESTORE_VIEW 
         * and the INVOKE_APPLICATION phases.
         * so we return ANY_PHASE and the selection will be done 
         * in the afterPhase method.
         * @ return PhaseId.ANY_PHASE
         */
	public PhaseId getPhaseId() {
            return PhaseId.ANY_PHASE; 
	}

}
